import { Response } from 'express';
import { AuthRequest } from '../../../middleware/auth';
import { userService } from '../../../services/userService';
import { logger } from '../../../config/logger';
import bcrypt from 'bcryptjs';
import { prisma } from '../../../lib/prisma';
import { getDefaultPermissions } from '../../../utils/permissions';


export async function getUsers(req: AuthRequest, res: Response): Promise<void> {
  try {
    // Проверка прав администратора
    if (!req.user?.permissions.isAdmin) {
      res.status(403).json({ error: 'Admin access required' });
      return
    }

    // Получение списка пользователей
    const users = await userService.getUsers();
    res.json(users);
  } catch {
    res.status(500).json({ error: 'Server error' });
  }
}

export async function createUser(req: AuthRequest, res: Response): Promise<void> {
  try {
    if (!req.user?.permissions.isAdmin) {
      logger.warn(`Non-admin user ${req.user?.id} attempted to create user`);
      res.status(403).json({ error: 'Admin access required' });
      return
    }

    const { email, password, displayName, avatarUrl } = req.body;

    // Check if user exists
    const existingUser = await prisma.user.findUnique({
      where: { email }
    });

    if (existingUser) {
      res.status(400).json({ error: 'User already exists' });
      return
    }

    // Hash password
    const hashedPassword = await bcrypt.hash(password, 10);

    // Create user with default permissions
    const user = await prisma.user.create({
      data: {
        email,
        password: hashedPassword,
        displayName,
        avatarUrl,
        permissions: getDefaultPermissions()
      },
      select: {
        id: true,
        email: true,
        displayName: true,
        avatarUrl: true,
        permissions: true
      }
    });

    logger.info(`User created successfully: ${user.id}`);
    res.status(201).json(user);
  } catch (error) {
    logger.error('Error creating user:', error);
    res.status(500).json({ error: 'Failed to create user' });
  }
}

export async function updateUser(req: AuthRequest, res: Response): Promise<void> {
  try {
    if (!req.user?.permissions.isAdmin) {
      logger.warn(`Non-admin user ${req.user?.id} attempted to update user`);
      res.status(403).json({ error: 'Admin access required' });
      return
    }

    const { id } = req.params;
    const { email, password, displayName, avatarUrl } = req.body;

    // Check if user exists
    const existingUser = await prisma.user.findUnique({
      where: { id }
    });

    if (!existingUser) {
      res.status(404).json({ error: 'User not found' });
      return
    }

    // Prepare update data
    const updateData: any = {
      email,
      displayName,
      avatarUrl
    };

    // Only update password if provided
    if (password) {
      updateData.password = await bcrypt.hash(password, 10);
    }

    // Update user
    const user = await prisma.user.update({
      where: { id },
      data: updateData,
      select: {
        id: true,
        email: true,
        displayName: true,
        avatarUrl: true,
        permissions: true
      }
    });

    logger.info(`User updated successfully: ${user.id}`);
    res.json(user);
  } catch (error) {
    logger.error('Error updating user:', error);
    res.status(500).json({ error: 'Failed to update user' });
  }
}

export async function updateUserPermissions(req: AuthRequest, res: Response): Promise<void> {
  try {
    // Проверка прав администратора
    if (!req.user?.permissions.isAdmin) {
      res.status(403).json({ error: 'Admin access required' });
      return
    }

    // Извлечение параметров из запроса
    const { id } = req.params;
    const { permissions } = req.body;

    // Обновление прав пользователя
    const user = await userService.updateUserPermissions(id, permissions);
    res.json(user);
  } catch {
    res.status(500).json({ error: 'Server error' });
  }
}

export async function deleteUser(req: AuthRequest, res: Response): Promise<void> {
  try {
    if (!req.user?.permissions.isAdmin) {
      logger.warn(`Non-admin user ${req.user?.id} attempted to delete user`);
      res.status(403).json({ error: 'Admin access required' });
      return
    }

    const { id } = req.params;

    // Check if user exists
    const existingUser = await prisma.user.findUnique({
      where: { id }
    });

    if (!existingUser) {
      res.status(404).json({ error: 'User not found' });
      return
    }

    // Delete user
    await prisma.user.delete({
      where: { id }
    });

    logger.info(`User deleted successfully: ${id}`);
    res.json({ message: 'User deleted successfully' });
  } catch (error) {
    logger.error('Error deleting user:', error);
    res.status(500).json({ error: 'Failed to delete user' });
  }
}